"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InvalidDurationValue = exports.InvalidDurationSyntax = exports.Duration = void 0;
const luxon_1 = require("luxon");
class Duration {
    constructor(duration) {
        const match = duration.match(Duration.durationRegex);
        if (!match) {
            throw new InvalidDurationSyntax(duration);
        }
        const unit = match.groups.unit;
        const value = Number(match.groups.value);
        if ((unit === 'mo' || unit === 'y') && value !== 1) {
            throw new InvalidDurationValue(duration);
        }
        this.duration = duration;
        this.durationValue = value;
        this.durationUnit = unit;
    }
    toDurationString() {
        const unit = this.durationUnit.toUpperCase();
        const value = this.durationValue;
        return unit === 'M' || unit === 'H' ? `PT${value}${unit}` : `P${value}${unit[0]}`;
    }
    getMilliSeconds() {
        const duration = luxon_1.Duration.fromISO(this.toDurationString());
        return duration.toMillis();
    }
    toLuxonDuration() {
        return luxon_1.Duration.fromISO(this.toDurationString()).toObject();
    }
    roundDate(dateToRound) {
        switch (this.durationUnit) {
            case 'mo':
                return luxon_1.DateTime.fromJSDate(dateToRound).startOf('month').toMillis();
            case 'y':
                return luxon_1.DateTime.fromJSDate(dateToRound).startOf('year').toMillis();
            default:
                return this.roundToClosestLower(dateToRound);
        }
    }
    getDurationUnit() {
        return this.durationUnit;
    }
    roundDateUp(dateToRound) {
        switch (this.durationUnit) {
            case 'mo': {
                const luxonDate = luxon_1.DateTime.fromJSDate(dateToRound);
                return luxonDate.equals(luxonDate.startOf('month'))
                    ? luxonDate.toMillis()
                    : luxon_1.DateTime.fromJSDate(dateToRound)
                        .plus({ month: 1 })
                        .startOf('month')
                        .toMillis();
            }
            case 'y':
                return luxon_1.DateTime.fromJSDate(dateToRound)
                    .startOf('year')
                    .plus({ year: 1 })
                    .toMillis();
            default:
                return this.roundToClosestUpper(dateToRound);
        }
    }
    roundToClosestLower(dateToRound) {
        const toClosest = this.getMilliSeconds();
        return Math.floor(dateToRound.getTime() / toClosest) * toClosest;
    }
    roundToClosestUpper(dateToRound) {
        const toClosest = this.getMilliSeconds();
        return Math.ceil(dateToRound.getTime() / toClosest) * toClosest;
    }
}
exports.Duration = Duration;
Duration.durationValidationRegex = /(^(\d+)(m|h|d|w)$)|(^(1)(mo|y)$)/;
Duration.durationRegex = /^(?<value>\d+)(?<unit>m|h|d|w|mo|y)$/;
class InvalidDurationSyntax extends Error {
    constructor(duration) {
        super(`Invalid duration syntax: ${duration}.`);
    }
}
exports.InvalidDurationSyntax = InvalidDurationSyntax;
class InvalidDurationValue extends Error {
    constructor(duration) {
        super(`Invalid duration value: ${duration}. Monthly or yearly durations can only have value of 1.`);
    }
}
exports.InvalidDurationValue = InvalidDurationValue;
//# sourceMappingURL=Duration.js.map